@@ -1,4 +1,4 @@
-/* $Id: color.c 4894 2014-05-16 10:34:05Z bens $ */
+/* $Id: color.c 5038 2014-06-30 17:49:53Z bens $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
@@ -155,18 +155,15 @@ void color_update(void)
     colortype *tmpcolor, *defcolor = NULL;
     regexlisttype *e;
 
-/* Var magicstring will stay NULL if we fail to get a magic result. */
-#ifdef HAVE_LIBMAGIC
-    const char *magicstring = NULL;
-    magic_t cookie = NULL;
-    struct stat fileinfo;
-#endif
-
     assert(openfile != NULL);
 
     openfile->syntax = NULL;
     openfile->colorstrings = NULL;
 
+    /* If the rcfiles were not read, or contained no syntaxes, get out. */
+    if (syntaxes == NULL)
+	return;
+
     /* If we specified a syntax override string, use it. */
     if (syntaxstr != NULL) {
 	/* If the syntax override is "none", it's the same as not having
@@ -262,6 +259,9 @@ void color_update(void)
 #ifdef HAVE_LIBMAGIC
 	/* Check magic if we don't have an answer yet. */
 	if (openfile->colorstrings == NULL) {
+	    struct stat fileinfo;
+	    magic_t cookie = NULL;
+	    const char *magicstring = NULL;
 #ifdef DEBUG
 	    fprintf(stderr, "No result from headerline either, trying libmagic...\n");
 #endif
